#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Select Faces by Size
#      By:	sdmitch
#   Usage:	Enter maximum and minimum values for face areas to be selected.
#	 Note:	If maximum size is less than minimum size and OR selection is made.
#			If maximum size is greater than minimum size and AND selection is made.
#			Since a number is expected for the sizes, an "empty" entry is not valid, enter 0 instead.
#    Date:	Jun 2011
#------------------------------------------------------------------------------------------------
require 'sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Select Faces by Size') { FBS.faces_by_size }
end
#
file_loaded(filename)
#
module FBS

	def FBS.faces_by_size
		mod=Sketchup.active_model; ents=mod.active_entities; @sel=mod.selection
		@sel.clear;@fbs_lt=100.0 if !@fbs_lt;@fbs_gt=200.0 if !@fbs_gt
		#	0=inches,1=feet,2=millimeters,3=centimeters,4=meters
		unit=Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
		case unit
			when 0:units_factor = 1.0;units="inches"
			when 1:units_factor = 12.0**2;units="feet"
			when 2:units_factor = 0.03937007874**2;units="mm"
			when 3:units_factor = 0.3937007874**2;units="cm"
			when 4:units_factor = 39.37007874**2;units="m"
		end
		ans=UI.inputbox(["Area <","Area >"],[@fbs_lt,@fbs_gt],"Select Faces By Size: ("+units+")")
		if !ans then
			Sketchup.send_action "selectSelectionTool:"
			return nil
		end
		@fbs_lt = ans[0];@fbs_gt = ans[1]
		@max = @fbs_lt * units_factor; @min = @fbs_gt * units_factor
		for e in ents
			if e.is_a? Sketchup::Face
				FBS.check_size(e)
			elsif e.is_a? Sketchup::Group
				for ge in e.entities
					FBS.check_size(ge) if ge.is_a? Sketchup::Face
				end
			elsif e.is_a? Sketchup::ComponentInstance
				for ce in e.definition.entities
					FBS.check_size(ce) if ce.is_a? Sketchup::Face
				end
			end
		end
	end

	def FBS.check_size(fe)
		#API Doc says area is return in current units but all I get is Sq inches.
		if @fbs_lt < @fbs_gt # OR case
			@sel.add fe if(fe.area < @max || fe.area > @min)
		else # AND case
			@sel.add fe if(fe.area < @max && fe.area > @min)
		end
	end
end